/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#include "Double.h"

#include <SimoxUtility/meta/type_name.h>

namespace armarx::aron::codegenerator::cpp::generator
{
    /* constructors */
    Double::Double(const type::Double& e) :
        detail::PrimitiveGenerator<type::Double, Double>(
            "double",
            "double",
            simox::meta::get_type_name<data::dto::AronDouble>(),
            simox::meta::get_type_name<type::dto::AronDouble>(),
            e)
    {
    }

    /* virtual implementations */
    std::pair<std::vector<std::pair<std::string, std::string>>, bool>
    Double::getCtorInitializers(const std::string& name) const
    {
        return {{{name, std::to_string(type.getDefaultValue())}}, true};
    }

    CppBlockPtr
    Double::getWriteTypeBlock(const std::string& typeAccessor,
                              const std::string& accessor,
                              const Path& p,
                              std::string& variantAccessor) const
    {
        CppBlockPtr b = std::make_shared<CppBlock>();
        std::string escaped_accessor = EscapeAccessor(accessor);
        variantAccessor = ARON_VARIANT_RETURN_ACCESSOR + "_" + escaped_accessor;

        b->addLine("auto " + variantAccessor + " = " + ARON_WRITER_ACCESSOR + ".writeDouble(" +
                   std::to_string(type.getDefaultValue()) + ", " +
                   conversion::Maybe2CppString.at(type.getMaybe()) + ", " + "armarx::aron::Path(" +
                   ARON_PATH_ACCESSOR + ", {" + simox::alg::join(p.getPath(), ", ") +
                   "})); // of " + typeAccessor);
        return b;
    }

    CppBlockPtr
    Double::getResetHardBlock(const std::string& cppAccessor) const
    {
        CppBlockPtr block_if_data = std::make_shared<CppBlock>();
        block_if_data->addLine(cppAccessor + " = 0.0;");
        return resolveMaybeResetHardBlock(block_if_data, cppAccessor);
    }

    CppBlockPtr
    Double::getResetSoftBlock(const std::string& cppAccessor) const
    {
        auto block_if_data = std::make_shared<CppBlock>();
        block_if_data->addLine(cppAccessor + " = 0.0;");
        return this->resolveMaybeResetSoftBlock(block_if_data, cppAccessor);
    }
} // namespace armarx::aron::codegenerator::cpp::generator
