/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <memory>
#include <string>
#include <unordered_map>

// Base class
#include "SpecializedVariant.h"

// ArmarX

namespace armarx::aron::type::detail
{
    template <typename AronTypeT, typename DerivedT, typename ValueT>
    class PrimitiveVariant : public SpecializedVariantBase<AronTypeT, DerivedT>
    {
    public:
        using Base = SpecializedVariantBase<AronTypeT, DerivedT>;

        using ValueType = ValueT;

        using Base::SpecializedVariantBase;

        virtual ~PrimitiveVariant() = default;

        /* virtual implementations */
        VariantPtr
        navigateAbsolute(const Path& path) const override
        {
            throw error::AronException(
                __PRETTY_FUNCTION__,
                "Could not navigate through a non container navigator. The input path was: " +
                    path.toString(),
                Variant::getPath());
        }

        std::vector<VariantPtr>
        getChildren() const override
        {
            return {};
        }

        size_t
        childrenSize() const override
        {
            return 0;
        }

        ValueType
        getDefaultValue() const
        {
            return this->aron->defaultValue;
        }

        void
        setDefaultValue(const ValueType& v)
        {
            this->aron->defaultValue = v;
        }

    protected:
    };
} // namespace armarx::aron::type::detail
